// Released under the MIT/X11 license
// http://www.opensource.org/licenses/mit-license.php
/*global require: false, exports: false, console: false */

"use strict";
var tabs = require("tabs");
var contextMenu = require("context-menu");

var testedStyles = ["overflow", "overflow-x", "overflow-y"];
var badStyleRE = new RegExp("auto|hidden");

function collectBadStylesheetRules(sh) {
    var rules = sh.cssRules, badRules = [];
    
    Array.forEach(rules, function (rule) {
        testedStyles.forEach(function (name) {
            if (badStyleRE.test(rule.style.getPropertyValue(name))) {
                badRules.push({
                    obj: rule,
                    prop: name
                });
            }
        });
    });
    
    return badRules;
}

function collectBadElements(elems) {
    var badElements = [];
    
    Array.forEach(elems, function (elem) {
        testedStyles.forEach(function (x) {
            if (badStyleRE.test(elem.style.getPropertyValue(x))) {
                badElements.push({
                    obj: elem,
                    prop: x
                });
            }
        });
    });
    
    return badElements;
}

function clearBadStuff(list) {
    list.forEach(function (e) {
        e.obj.style.removeProperty(e.prop);
    });
}

exports.main = function main(options, callbacks) {
    tabs.onReady = function (tab) {
        var bStuff = [], menuItem = null;
        
        Array.forEach(tab.contentDocument.styleSheets, function (sh) {
            if (!sh.disabled) {
                bStuff = bStuff.concat(collectBadStylesheetRules(sh));
            }
        });

        // we should use tab.contentDocument.querySelectorAll somehow, but
        // not sure how.
        bStuff =  bStuff.concat(collectBadElements(tab.contentDocument.getElementsByTagName("*")));
        
        if (bStuff.length > 0) {
            menuItem = contextMenu.Item({
                label: "Clear Bad stuff",
                onClick: function (contextObj, item) {
                    clearBadStuff(bStuff);
                }
            });
            contextMenu.add(menuItem);
        }
    };
};